"""This module only contains the Node class.

Author: Thomas Oberbichler
"""

import numpy as np

class Node(object):
    """Three dimensional Node providing Dofs for displacements.

    Attributes
    ----------
    id : int or str
        Unique ID.
    reference_x : float
        Reference X coordinate.
    reference_y : float
        Reference Y coordinate.
    reference_z : float
        Reference Z coordinate.
    x : float
        Actual X coordinate.
    y : float
        Actual Y coordinate.
    z : float
        Actual Z coordinate.
    u : float
        Displacement in x direction.
    v : float
        Displacement in y direction.
    w : float
        Displacement in z direction.
    """

    def __init__(self, id, x, y, z):
        """Create a new node.

        Parameters
        ----------
        id : int or str
            Unique ID of the node.
        x : float
            Initial X coordinate of the node.
        y : float
            Initial Y coordinate of the node.
        z : float
            Initial Z coordinate of the node.
        """
        self.id = id
        self.x = x
        self.y = y
        self.z = z
        self.reference_x = x
        self.reference_y = y
        self.reference_z = z

    @property
    def u(self):
        return self.x - self.reference_x

    @u.setter
    def u(self, value):
        self.x = self.reference_x + value

    @property
    def v(self):
        return self.y - self.reference_y

    @v.setter
    def v(self, value):
        self.y = self.reference_y + value
    
    @property
    def w(self):
        return self.z - self.reference_z

    @w.setter
    def w(self, value):
        self.z = self.reference_z + value

    def get_reference_location(self):
        """Location of the node in the reference configuration.

        Returns
        -------
        location : ndarray
            Numpy array containing the reference coordinates X, Y and Z.
        """
        x = self.reference_x
        y = self.reference_y
        z = self.reference_z

        return np.array([x, y, z], dtype=float)

    def get_actual_location(self):
        """Location of the node in the actual configuration.

        Returns
        -------
        location : ndarray
            Numpy array containing the actual coordinates X, Y and Z.
        """
        x = self.x
        y = self.y
        z = self.z

        return np.array([x, y, z], dtype=float)

    def get_displacement(self):
        """Displacement of the node in the actual configuration.

        Returns
        -------
        displacement : ndarray
            A numpy array containing the displacements u, v and w.
        """
        return self.get_reference_location() - self.get_actual_location()

    def update(self, dof_type, value):
        """
        .. note:: Deprecated
                  Use `SetDofState` instead
        """
        raise DeprecationWarning('Use `SetDofState` instead')
        if dof_type == 'u':
            self.u = value
        elif dof_type == 'v':
            self.v =  value
        elif dof_type == 'w':
            self.w = value
        else:
            raise RuntimeError('Node has no Dof of type {}'.format(dof_type))

    def get_dof_state(self, dof_type):
        """Get the current value of the given dof type.

        Parameters
        ----------
        dof_type : string
            Type of the dof.

        Returns
        -------
        value : float
            The current value of the dof type

        Raises
        ------
        AttributeError
            If `dof_type` does not exist.
        """
        if dof_type == 'u':
            return self.u
        if dof_type == 'v':
            return self.v
        if dof_type == 'w':
            return self.w

        raise AttributeError('Node has no dof of type \'{}\''.format(dof_type))

    def set_dof_state(self, dof_type, value):
        """Update the node according to the value of the given dof type.

        Parameters
        ----------
        dof_type : string
            Type of the Dof.
        value : float
            The value of the given dof.

        Raises
        ------
        AttributeError
            If `dof_type` does not exist.
        """
        if dof_type == 'u':
            self.u = value
        elif dof_type == 'v':
            self.v = value
        elif dof_type == 'w':
            self.w = value
        else:
            raise AttributeError('Node has no dof of type \'{}\''.format(dof_type))



"""
NFEM teaching tool

A module for the non linear static analysis of 3D truss problems.
A light weight object oriented FEM code and some usefull postprocessing tools.
"""

import sys

__version__ = '1.7.2'

print(f"""
--------------------------------------------------------------------------------

                       _   ________________  ___
                      / | / / ____/ ____/  |/  /
                     /  |/ / /_  / __/ / /|_/ /
                    / /|  / __/ / /___/ /  / /
                   /_/ |_/_/   /_____/_/  /_/  Teaching Tool

  Authors:   Armin Geiser, Aditya Ghantasala, Thomas Oberbichler, Klaus Sautter
             Mahmoud Zidan
  Copyright: © 2018-2020 TUM Statik
  Version:   {__version__}

  This is a teaching tool! All results without warranty.

--------------------------------------------------------------------------------
""")

if sys.version_info < (3, 5):
    raise RuntimeError("The nfem module requires at least Python 3.5!")

from .model import Model
from .node import Node
from .truss import Truss
from .assembler import Assembler

from .newton_raphson import newton_raphson_solve
from .path_following_method import LoadControl
from .path_following_method import DisplacementControl
from .path_following_method import ArcLengthControl

from .visualization import interact
from .visualization import show_load_displacement_curve, show_animation, show_deformation_plot
from .visualization import Plot2D, Animation3D, DeformationPlot3D

from .bracketing import bracketing

def test():
    import unittest
    suite = unittest.TestLoader().discover('.', pattern='test*')
    unittest.TextTestRunner(verbosity=2).run(suite)



from .interactive import interact #needs to be imported first, because it sets the matplotlib backend
from .plot import show_load_displacement_curve, show_animation, show_deformation_plot
from .plot import Plot2D, Animation3D, DeformationPlot3D
